AJS.toInit(function() {
	console.log("Initializing");
	var baseUrl = AJS.$("meta[name='application-base-url']").attr("content");

	var nextFieldIndex = 0;

	// Setting the baseUrl meta tag in confluence is not working, but there is a provided
	// meta-tag called confluence-base-url that does the job.
	if(typeof baseUrl === "undefined") {
		baseUrl = AJS.$("meta[name='confluence-base-url']").attr("content");
	}

	function disableForm() {
		AJS.$(".disableable :input").prop('disabled',true);
		AJS.$("#waitmessage").show();
	}

	function enableForm() {
		AJS.$(".disableable :input").prop('disabled',false);
		AJS.$("#waitmessage").hide();
		window.scrollTo(0, 0);
	}

	function populateForm() {
		console.log("Loading configuration data");
		disableForm();
		AJS.$.ajax({
			url: baseUrl + "/rest/samlsso-admin/1.0/",
			dataType: "json",
			success: function(config) {
				console.log("Filling form fields");
				fillFields(config);

				AJS.$("#licenseMessage").text(config.licenseMessage);
				if(config.licensed) {
					AJS.$("#licenseMessage").removeClass("error");
					AJS.$("#licenseMessage").addClass("success");
				} else {
					AJS.$("#licenseMessage").addClass("error");
					AJS.$("#licenseMessage").removeClass("success");    	
				}
				enableForm();
			},
			error: function() {
				console.log("Loading configuration data failed");
				AJS.$("#aui-message-bar").empty(); 
				AJS.messages.error({
					title: "Load failed",
					body: "Loading settings failed!",
					closeable: false
				});
				enableForm();
			}
		});
	}

	/*
	 * Writes the config into the form elements. 
	 * The value is assigned to the form element having the key as name attribute.
	 */
	function fillFields(config) {
		console.log("Loading data into the forms")
		AJS.$.each(config, function(k,v) {
			if(k == 'idpConfigurations') {
				addIdPConfigurationFields(v);
			} else {
				var formElement = AJS.$("*[name=" + k + "]");
				if (formElement.length == 0) {
					console.log("Found no element with name " + k);
				} else {
					if(formElement.attr("type") == "checkbox") {
						formElement.prop('checked',v);
					} else  {
						if(formElement.hasClass('newlineseparated')) {
							v = replaceCommasWithNewlines(v);
						}	
						formElement.val(v);
					}
				} 
			}
		});
	}

	///
	/// Creates the form fields for the IdP configuration
	///
	function createIdpFields(index) {
		console.log("Creating Idp fields" + index );

		var container = AJS.$("<div>",{"id":"idp"+index,"class":"idpContainer withBorder aui long-label"});

		var heading = (AJS.$("<h3>",{text:"Identity Provider "}));
		var idSpan = AJS.$("<span>",{"id":"id-idp"+index, text:"NEW"});
		heading.append(idSpan);

		var deleteButton =  AJS.$("<button>",{"class":"aui-button aui-button-subtle",title:"Delete",click:function(e){ container.remove()}})
		deleteButton.prepend('<span class="aui-icon aui-icon-small aui-iconfont-delete">delete</span></button>')
		heading.append(deleteButton);

		container.append(heading);
			
		var fn_idpurl = "IdP POST Binding URL";
		var fd_idpurl = 'SAML-requests are sent to this URL on the Identity Provider using the HTTP-POST binding.<br/> \
			If you have metadata as XML, copy the Location URL from the tag looking like <br/>\
			<i>&lt;SingleSignOnService Binding=&quot;urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST&quot; Location=&quot;<b>https://some.example/adfs/ls/</b>&quot;/&gt;</i><br/>';	
		container.append(AJS.$(
				'<div class = "field-group">\
				<label for="url-idp' + index + '">' + fn_idpurl + '</label>\
				<input type="text" class="text long-field" id="url-idp' + index + '" name="url-idp'+index+'">\
				<div class="description">' + fd_idpurl + '</div>\
			</div>'));	
		
		var fn_issuer = "IdP Entity ID";
		var fd_issuer = 'To match incoming SAML-messages to a specific IdP, the message\'s <i>&lt;Issuer&gt;</i>-value is matched against this URL.<br/>\
			If you have metadata as XML, use the entityID value found in the EntityDescriptor: \
			<i>&lt;EntityDescriptor ... entityID=&quot;<b>http://some.example/adfs/services/trust</b>&quot; ...&gt;</i><br/>This field is mandatory if you have more than one IdP.'; 
		container.append(AJS.$(
				'<div class = "field-group">\
					<label for="issuer-idp' + index + '">' + fn_issuer +'</label>\
					<input type="text" class="text long-field" id="issuer-idp' + index + '" name="issuer-idp'+index+'">\
					<div class="description">' + fd_issuer + '</div>\
				</div>'));
		
				
		var fn_certificate = "IdP Token Signing Certificate";
		var fd_certificate = 'SAML responses from the IdP should be digitally signed. This signature is validated against the certificate specified here.<br/>\
			Paste the Base64-encoded IdP Token Signing Certificate including the BEGIN and END CERTIFICATE lines.<br/>\
			<b>You can leave this empty, but this will disable the validation and all incoming SAML-messages will be trusted. <i>This is a security risk, so we strongly recommend to configure a certificate!</i></b>';
		
		
		var fgCertificate = AJS.$("<div>",{"class":"field-group"});
		fgCertificate.append(AJS.$("<label>",{"for":"certificate-idp"+index,text:fn_certificate}));
		var certField = AJS.$("<textarea>",{"class":"textarea full-width-field",rows:"3",cols:"60",id:"certificate-idp"+index,name:"certificate-idp"+index});
		fgCertificate.append(certField);

		var certStatus = AJS.$("<span>",{"class":"aui-lozenge aui-lozenge-current","text":"not set","id":"certstatus-idp"+index});

		var readableCertContainer = AJS.$("<div>",{"class":"aui-expander-content","id":"decoded-cert-cont-idp"+index});
		var readableCertDiv = AJS.$("<div>",{"class":"readableCert","id":"decoded-cert-container-idp"+index,"text":"no certificate set"});
		readableCertContainer.append(readableCertDiv);

		certField.on("change paste",function(e) { checkCertificate(certField, certStatus, readableCertDiv) });

		fgCertificate.append(certStatus);
		fgCertificate.append(AJS.$("<a>",{
			id:"decoded-cert-container-trigger-idp"+index,
			text:" Show details",
			"data-replace-text":" Hide details",
			"class":"aui-expander-trigger",
			"aria-controls":"decoded-cert-cont-idp"+index
		}));
		
		var certDesc = AJS.$("<div>",{"class":"description",html:fd_certificate});

		fgCertificate.append(certDesc);

				
		var fgReadableCert = AJS.$("<div>",{"class":"field-group"})
		readableCertDiv.append(fgReadableCert);
		
		fgCertificate.append(readableCertContainer);
		container.append(fgCertificate);
		
		var fn_useridAttribute = "Userid Attribute";
		var fd_useridAttribute = 'You can specifiy Attribute in the SAML Response to read the userid from. Leave this empty to use the <i>NameID</i>-value from the response\'s <i>&lt;Subject&gt;</i>-tag.</div>'
		container.append(AJS.$(
				'<div class = "field-group">\
					<label for="useridAttribute-idp' + index + '">'+ fn_useridAttribute + '</label>\
					<input type="text" class="text long-field" id="useridAttribute-idp' + index + '" name="useridAttribute-idp'+index+'">\
					<div class="description">'+ fd_useridAttribute + '</div>'));
		
		var fd_useridtransform = 'Regex and replacement to modify the userid delivered from IdP. Leave empty to disable transformation.</br>\
			Capturing groups can be referenced in the replacement with <i>$1</i>,<i>$2</i>...</br>\
			<b>Example:</b> To transform <i>gonzo@example.com</i> to <i>gonzo</i> use <i>(.*)@.*</i> as regex and <i>$1</i> as replacement.'
			
		container.append(AJS.$(
			'<div class = "field-group">\
				<label for="useridTransformationRegex-idp' + index + '">Userid Transformation</label>\
				<input type="text" class="text" id="useridTransformationRegex-idp' + index + '" name="useridTransformationRegex-idp' + index + '">\
				<input type="text" class="text" id="useridTransformationReplacement-idp' + index + '" name="useridTransformationReplacement-idp' + index + '">\
				<div class="description">' + fd_useridtransform + '</div>\
			</div>'));
		
		container.append(AJS.$(
				'<div class = "field-group">\
					<label for="relayStateParameterName-idp' + index + '">Relay State parameter name</label>\
					<input type="text" class="text long-field" id="relayStateParameterName-idp' + index + '" name="relayStateParameterName-idp'+index+'">\
					<div class="description">URL Parameter used to pass the original URL called by the user. In most cases, this is RelayState, so just leave the default.</div>\
				</div>'));
		
		var fn_omitRequestedAuthnContext = "Omit AuthnContext";
		var fd_omitRequestedAuthnContext = 'By default, this RequestedAuthnContext is included in the SAML-request:<br/>\
		<i>&lt;saml2p:RequestedAuthnContext xmlns:saml2p=&quot;urn:oasis:names:tc:SAML:2.0:protocol&quot; Comparison=&quot;minimum&quot;&gt;<br/>\
		  &lt;saml:AuthnContextClassRef xmlns:saml=&quot;urn:oasis:names:tc:SAML:2.0:assertion&quot;&gt;urn:oasis:names:tc:SAML:2.0:ac:classes:Password&lt;/saml:AuthnContextClassRef&gt;<br/>\
		&lt;/saml2p:RequestedAuthnContext&gt;</i><br/>\
		Check this box to omit this.'
		
		container.append(AJS.$(
				'<div class = "field-group">\
					<label for="omitRequestedAuthnContext-idp' + index + '">'+ fn_omitRequestedAuthnContext +'</label>\
					<input type="checkbox" class="checkbox" id="omitRequestedAuthnContext-idp' + index + '" name="omitRequestedAuthnContext-idp' + index + '">\
					<div class="description">' + fd_omitRequestedAuthnContext + '</div>\
				</div>'));
		
		var fn_name = "IdP Name";
		var fd_name = "Choose a unique name. This name is shown in the IdP selection page.";	
		container.append(AJS.$(
				'<div class = "field-group">\
				<label for="name-idp' + index + '">' + fn_name + '</label>\
				<input type="text" class="text long-field" id="name-idp' + index + '" name="name-idp'+index+'">\
				<div class="description">' + fd_name + '</div>\
			</div>'));		
		
		var fn_description = "IdP Description";
		var fd_description = 'Description for this IdP. This value is shown on the IdP selection page.';
		container.append(AJS.$(
				'<div class = "field-group">\
					<label for="description-idp' + index + '">' + fn_description +'</label>\
					<input type="text" class="text long-field" id="description-idp' + index + '" name="description-idp'+index+'">\
					<div class="description">' + fd_description + '</div>\
				</div>'));
		
		return container;	
	}

	function fillIdPConfigurationFields(index,config) {
		console.log("Filling idp fields " + index);

		var idpid = config["id"];
		AJS.$("#id-idp"+index).text(idpid);

		for(var key in config) {
			var value = config[key];
			var elementId = key + "-idp" + index;
			var formElement = AJS.$("*[id="+ elementId + "]");
			if (formElement.length == 0) {
				console.log("Found no element with id " + elementId);
			} else {
				if(formElement.attr("type") == "checkbox") {
					formElement.prop('checked',value);
				} else  {
					if(formElement.hasClass('newlineseparated')) {
						value = replaceCommasWithNewlines(value);
					}
					formElement.val(value);
					
				}
			} 		
		}	
		checkCertificate(AJS.$("#certificate-idp"+index), AJS.$("#certstatus-idp"+index), AJS.$("#decoded-cert-container-idp"+index))
	}

	function addIdPConfigurationFields(idpConfigurations) {	
		var container = AJS.$("#idpConfigurationDiv");
		container.empty();	
		if(idpConfigurations.length == 0) {
			console.log("No IdPs in configuration, creating an empty field set");
			nextFieldIndex = 1;
			addNewIdPFields();
		} else {
			for(var i = 0, size = idpConfigurations.length; i<size;i++) {
				var current = idpConfigurations[i];
				container.append(createIdpFields(i));
				fillIdPConfigurationFields(i,current);
			}
			nextFieldIndex = idpConfigurations.length;
		}
	}

	function addNewIdPFields() {
		nextFieldIndex++;
		var container = AJS.$("#idpConfigurationDiv");
		container.append(createIdpFields(nextFieldIndex));
	}

	function readIdPFields() {
		var idpData = [];
		AJS.$("#idpConfigurationDiv").children().each( function(index,element){
			var data = {}			
			var idpId = AJS.$(element).find("span[id*='id-idp']").first().text();
			if(idpId == "NEW") {
				idpId = -1;
			} 
			data['id'] = idpId;

			AJS.$(element).find("input:not([type='submit'],[type='reset']),textarea").each( function(index,formElement) {
				var formId = AJS.$(formElement).attr("id");
				var key = formId.replace(/-idp.*$/,'');
				var value;
				if(AJS.$(formElement).attr("type") == "checkbox") {
					value = AJS.$(formElement).prop('checked');	// Checkboxes need to be checked for the checked attribute
				} else {
					value = AJS.$(formElement).val(); // from all other elements the value can be retrieved with the val() function 
					if(AJS.$(formElement).hasClass('newlineseparated')) {
						value = replaceNewlinesWithCommas(value);
					}
				}
				data[key] = value;		
			});
			idpData.push(data);
		});
		return idpData;
	}

	/*
	 * Reads the data from the form field into a JSON object.
	 * The form element's name attribute is used as key.
	 */
	function readFields() {
		var returnData = {};
		// Iterate through all input elements and textareas expect reset and submit buttons
		returnData["idpConfigurations"] = readIdPFields();


		AJS.$("#readTheseFields").find("input:not([type='submit'],[type='reset']),textarea").each( function(index,formElement) {
			var name = AJS.$(formElement).attr("name");
			var value;
			if(AJS.$(formElement).attr("type") == "checkbox") {
				value = AJS.$(formElement).prop('checked');	// Checkboxes need to be checked for the checked attribute
			} else {
				value = AJS.$(formElement).val(); // from all other elements the value can be retrieved with the val() function 
				if(AJS.$(formElement).hasClass('newlineseparated')) {
					value = replaceNewlinesWithCommas(value);
				}
			}
			returnData[name] = value;	
		});
		return returnData;	
	}

	function updateConfig() {

		disableForm();
		var ajaxdata = readFields();

		AJS.$.ajax({
			url: baseUrl + "/rest/samlsso-admin/1.0/",
			type: "PUT",
			contentType: "application/json; charset=utf-8",
			data: JSON.stringify(ajaxdata),
			processData: true,
			success: function () { 
				AJS.$("#aui-message-bar").empty(); 
				AJS.messages.success({
					title: "Saved",
					body: "Settings successfully saved.",
					closeable: false
				});
				populateForm();
			},
			error: function(result) { 
				var errorMessages = AJS.$.parseJSON(result.responseText);
	
				var errorList = AJS.$('<ul>',{"class":"validationMessages"});
				for(i=0; i<errorMessages.length; i++) {
					console.log(errorMessages[i]);
					errorList.append(AJS.$("<li>",{text:errorMessages[i]}));
				}
				var msgBody = AJS.$("<div>");
				msgBody.append(errorList);
				
				AJS.$("#aui-message-bar").empty(); 
				AJS.messages.error({
					title: "Save failed",
					body: msgBody.html(),
					closeable: false
				}); 
				enableForm();
			}
		});  
	}

	function checkCertificate(certField, lozenge, readableDiv) {
		var certString = certField.val().trim();

		if(certString == "") {
			AJS.$(lozenge).removeClass();
			AJS.$(lozenge).addClass("aui-lozenge aui-lozenge-current");
			AJS.$(lozenge).text("not set");
		} else {
			var spinner = AJS.$("<span>",{"class":"aui-icon aui-icon-wait"});
			AJS.$(lozenge).removeClass();
			AJS.$(lozenge).addClass("aui-lozenge");
			AJS.$(lozenge).text("checking ");
			AJS.$(lozenge).append(spinner);

			AJS.$.ajax({
				url: baseUrl + "/rest/samlsso-admin/1.0/checkCertificate",
				type: "PUT",
				contentType: "text/plain; charset=utf-8",
				data: certString,
				success: function(result) { 
					AJS.$(lozenge).empty();
					AJS.$(lozenge).removeClass();
					AJS.$(lozenge).addClass("aui-lozenge aui-lozenge-success");
					AJS.$(lozenge).text("OK");
					AJS.$(readableDiv).html(escapeHtmlAndNewlines(result));

				},
				error: function(result) { 
					AJS.$(lozenge).empty();
					AJS.$(lozenge).removeClass();
					AJS.$(lozenge).addClass("aui-lozenge aui-lozenge-error");
					AJS.$(lozenge).text("Validity check failed");
					AJS.$(readableDiv).text(result.responseText);
				},
				statusCode: {
					400: function(result) {
						AJS.$(lozenge).empty();
						AJS.$(lozenge).removeClass();
						AJS.$(lozenge).addClass("aui-lozenge aui-lozenge-error");
						AJS.$(lozenge).text("Invalid");
						AJS.$(readableDiv).text(result.responseText);	
					}
				}
				
			});  
		}
	}

	populateForm();

	AJS.$("#admin").submit(function(e) {
		e.preventDefault();
	});

	AJS.$("#saveButton").click(function(e) {
		updateConfig();
	});

	AJS.$("#cancelButton").click(function(e) {
		AJS.$("#aui-message-bar").empty(); 
		populateForm();
	});

	AJS.$("#addIdPButton").click(function(e) {
		addNewIdPFields();
	});

}); // AJS.toInit

/*
 *  Escape newline chars for being transmitted with JSON over the wire
 */
function escapeNewLineChars(valueToEscape) {
	if (valueToEscape != null && valueToEscape != "") {
		return valueToEscape.replace(/\n/g, "\\n");
	} else {
		return valueToEscape;
	}
}

function replaceNewlinesWithCommas(text) {
	var replaced = text.replace('\n',',');
	return replaced;
}

function replaceCommasWithNewlines(text) {
	var replaced = text.replace(',','\n');
	return replaced;
}

function escapeHtmlAndNewlines(text) {
	var entityMap = {
	    "&": "&amp;",
	    "<": "&lt;",
	    ">": "&gt;",
	    '"': '&quot;',
	    "'": '&#39;',
	    "/": '&#x2F;'
	 };
	
	if (text != null && text != "") {
		var htmlEscaped = text.replace(/[&<>"'\/]/g, function (s) { return entityMap[s]; });
		return htmlEscaped.replace(/\n/g, "</br>");
	} else {
		return "";
	}
}
